#! /usr/bin/perl -w

use Device::SerialPort;
use Getopt::Long;

$version=0.1;

########################################
#
# GPL - http://www.equinoxefr.org
#
########################################

$DEVICE="/dev/ttyHS3";
my $port=Device::SerialPort->new($DEVICE);

sub send_cmd ($)
{
    my $cmd = shift;
    $port->write("$cmd\r");
    my $result="";
    $result=$port->input until ($result =~ /.*OK./ || $result =~ /.*ERROR./);
    $result=~ s/OK\r\n//g;
    return $result;
}

sub setup_port
{
    $port->baudrate(19200);
    $port->parity("odd");
    $port->databits(8);
    $port->stopbits(1);
}

sub signal_strength
{
   my ($signal)= (&send_cmd("AT\+CSQ") =~ /\+CSQ\: (\d+),.*/i);
   return ($signal);
}

sub network_capabilities
{
    my $capa="";
    my $result = &send_cmd("AT+WS46=?");
    if ( $result =~ /12/)
    {
	$capa .= "GSM\n"
    }
    if ( $result =~ /22/)
    {
	$capa .= "3G\n"
    }
    if ( $result =~ /25/)
    {
	$capa .= "GPRS\n"
    }

    return $capa;
}

sub hardware_infos
{
    my ($hw_info) = &send_cmd("ATI");
    my ($imei,$serial) = (&send_cmd("AT+CGSN") =~ m/(.*?),(.*?)\r\n/);
    return "$hw_info IMEI: $imei\n Serial Number: $serial\n";
}

my $hw;
my $net;
my $ver;
my $help;
my $pin;
my $connect;
my $disconnect;
my $apn="orange";


GetOptions ('help' => \$help, 'version' => \$ver, 'hw' => \$hw, 'net' => \$net, 'pin=s' => \$pin, 'connect' => \$connect, 'disconnect' => \$disconnect, 'apn=s' => \$apn);


&setup_port();

if (defined($help))
{
    print "Usage:\n";
    print "\t--help: this message :-)\n";
    print "\t--version: version of $0\n";
    print "\t--hw: information about your 3G hardware\n";
    print "\t--net: information about network capabilities\n";
    print "\t--pin=XXXX: PIN number\n";

}

if (defined($ver))
{
    print "Version: $version\n";
}

if (defined($hw))
{

    print "Hardware informations:\n";
    print &hardware_infos;
}

if (defined($net))
{
    printf ("Signal Quality:%s\n",&signal_strength());
    printf ("Network capabilities:\n%s",&network_capabilities);
}

if (defined ($connect))
{
    print "Connecting\n";
    &send_cmd("ATZ");    
    printf ("Signal Quality:%s\n",&signal_strength());
    if ( !defined($pin))
    {
	print "No PIN provided, enter it now:";
	$pin=<STDIN>;
	chomp($pin);
    }
    #&send_cmd("ABORT ERROR");
    #&send_cmd("TIMEOUT 10");
    
    my $result = &send_cmd("AT+CPIN=$pin");
    if ($result =~ /ERROR/)
	{
	    print "Warning ERROR in PIN:\n$result\n";
	    exit -1;
	}
    $result = &send_cmd("AT+COPS=0");
    $result = &send_cmd("AT+COPS=?");
    print "$result\n";
}


if (defined ($disconnect))
{
    print "Disconnecting\n";
}

undef $port;
